pro lattice::display
print, 'direct params',self.a,self.b,self.c,self.alpha,self.beta,self.gamma, format='(a, 5f7.3,5f7.3,5f7.3,5f7.3,5f7.3,5f7.3)'
print, 'reciprocal params', self.astar,self.bstar,self.cstar,self.alphastar,self.betastar,self.gammastar, format='(a,5f7.3,5f7.3,5f7.3,5f7.3,5f7.3,5f7.3)'
end

function lattice::metrictensor
g=dblarr(3,3)
dtor=!pi/180
a=self.astar
b=self.bstar
c=self.cstar
alphar=self.alphastar*dtor
betar=self.betastar*dtor
gammar=self.gammastar*dtor
g=[[a^2,a*b*cos(gammar),a*c*cos(betar)], $
   [a*b*cos(gammar),b^2,b*c*cos(alphar)], $
   [a*c*cos(betar),b*c*cos(alphar),c^2]]
return, g
end

function lattice::scalar,x,y
    g=self->metrictensor()
    prod=0
    for i=0,2 do begin
        for j=0,2 do begin
        	prod=prod+x(i)*g(i,j)*y(j)

        end
    end

    return, prod
end;scalar

function lattice::modvec,x
    modx=sqrt(self->scalar(x,x))
    return, modx
end;modvec


function lattice::cross,x,y
z=[0.,0.,0.]
z(0)=x(1)*y(2)-y(1)*x(2)
z(1)=x(2)*y(0)-y(2)*x(0)
z(2)=-x(1)*y(0)+y(1)*x(0)
return, z
end


function lattice::standardsystem,orient1,orient2


    print,'lattice::standardsystem'
    print,orient1

    x=orient1/self->modvec(orient1)
    proj=self->scalar(orient2,x)
    y=orient2;
    y=y-x*proj ; element by element
    y=y/self->modvec(y)
    z=self->cross(x,y)
    proj=self->scalar(z,y)
    z=z-y*proj
    z=z/self->modvec(z)

    self.x=x
    self.y=y
    self.z=z

;print, x,y, z
return, 1
end




function lattice::tmatrix, orient1,orient2,q


    dum=self->standardsystem(orient1,orient2)
    q=q/self->modvec(q)
    xq=self->scalar(self.x,q)
    yq=self->scalar(self.y,q)
    zq=0
    tmatrix=dblarr(4,4)
    tmatrix(0,0)=xq
    tmatrix(1,1)=xq
    tmatrix(0,1)=yq
    tmatrix(1,0)=-yq
    tmatrix(2,2)=1
    tmatrix(3,3)=1
    return, tmatrix
end;tmatrix

function lattice::rtransform, orient1,orient2,q,r
tmatrix=self->tmatrix(orient1,orient2,q)
rt=transpose(tmatrix)##(r##tmatrix)
return, rt

end

pro lattice::setLattice,a,b,c,alpha,beta,gamma,orient1,orient2

    self->setproperty,  la=a,$
                		lb=b,$
                		lc=c,$
                		alpha=alpha,$
                		beta=beta,$
            	    	gamma=gamma

    ret = self->standardsystem(orient1,orient2)

end;calculate
pro lattice::getproperty,$
		la=la,$
		lb=lb,$
		lc=lc,$
		alpha=alpha,$
		beta=beta,$
		gamma=gamma,$
;		astar=astar,$
;		bstar=bstar,$
;		cstar=cstar,$
;		alphastar=alphastar,$
;		betastar=betastar,$
;		gammastar=gammastar,$
		x=x,$
		y=y,$
		z=z

		if arg_present(la) ne 0 then la=self.la
		if arg_present(lb) ne 0 then lb=self.lb
		if arg_present(lc) ne 0 then lc=self.lc
		if arg_present(alpha) ne 0 then alpha=self.alpha
		if arg_present(beta) ne 0 then beta=self.beta
		if arg_present(gamma) ne 0 then gamma=self.gamma
		if arg_present(x) ne 0 then x=self.x
		if arg_present(y) ne 0 then y=self.y
		if arg_present(z) ne 0 then z=self.z





end;getproperty
pro lattice::setproperty,$
		la=la,$
		lb=lb,$
		lc=lc,$
		alpha=alpha,$
		beta=beta,$
		gamma=gamma,$
;		astar=astar,$
;		bstar=bstar,$
;		cstar=cstar,$
;		alphastar=alphastar,$
;		betastar=betastar,$
;		gammastar=gammastar,$
		x=x,$
		y=y,$
		z=z

        help,la
        help,lb
        help,lc

		if n_elements(la) ne 0 then self.a=la
		if n_elements(lb) ne 0 then self.b=lb
		if n_elements(lc) ne 0 then self.c=lc
		if n_elements(alpha) ne 0 then self.alpha=alpha
		if n_elements(beta) ne 0 then self.beta=beta
		if n_elements(gamma) ne 0 then self.gamma=gamma
;		if n_elements() ne 0 then self.astar=astar      ;CALCULATED IN INIT
;		if n_elements() ne 0 then self.bstar=bstar
;		if n_elements() ne 0 then self.cstar=cstar
;		if n_elements() ne 0 then self.alphastar=alphastar
;		if n_elements() ne 0 then self.betastar=betastar
;		if n_elements() ne 0 then self.gammastar=gammastar
		if n_elements(x) ne 0 then self.x=x
		if n_elements(y) ne 0 then self.y=y
		if n_elements(z) ne 0 then self.z=z

        self->reciplattice;,self.a,self.b,self.c,self.alpha,self.beta,self.gamma


end


pro lattice::reciplattice

    a = self.a
    b = self.b
    c = self.c
    alpha = self.alpha
    beta = self.beta
    gamma = self.gamma

	alphar=alpha*!dtor
	betar=beta*!dtor
	gammar=gamma*!dtor



	V=a*b*c*sqrt(1-cos(alphar)^2-cos(betar)^2-cos(gammar)^2+2*cos(alphar)*cos(betar)*cos(gammar))
	self.astar=b*c*sin(alphar)/V
	self.bstar=a*c*sin(betar)/V
	self.cstar=a*b*sin(gammar)/V
	self.alphastar=acos((cos(betar)*cos(gammar)-cos(alphar))/sin(betar)/sin(gammar))/!dtor
	 self.betastar=acos((cos(alphar)*cos(gammar)-cos(betar))/sin(alphar)/sin(gammar))/!dtor
	self.gammastar=acos((cos(alphar)*cos(betar)-cos(gammar))/sin(alphar)/sin(betar))/!dtor


end;reciplattice

function lattice::init, a, b, c, alpha, beta, gamma
	;dtor=!pi/180.
	if (n_params() ne 6) then return, 0
;	alphar=alpha*dtor
;	betar=beta*dtor
;	gammar=gamma*dtor
;	V=a*b*c*sqrt(1-cos(alphar)^2-cos(betar)^2-cos(gammar)^2+2*cos(alphar)*cos(betar)*cos(gammar))
;	astar=b*c*sin(alphar)/V
;	bstar=a*c*sin(betar)/V
;	cstar=a*b*sin(gammar)/V
;	alphastar=acos((cos(betar)*cos(gammar)-cos(alphar))/sin(betar)/sin(gammar))
;	 betastar=acos((cos(alphar)*cos(gammar)-cos(betar))/sin(alphar)/sin(gammar))
;	gammastar=acos((cos(alphar)*cos(betar)-cos(gammar))/sin(alphar)/sin(betar))

	self.a=a
	self.b=b
	self.c=c
	self.alpha=alpha
	self.beta=beta
	self.gamma=gamma

    self->reciplattice;,a,b,c,alpha,beta,gamma

;	self.astar=astar
;	self.bstar=bstar
;	self.cstar=cstar
;	self.alphastar=self.alphastar/!dtor
;	self.betastar=self.betastar/!dtor
;	self.gammastar=self.gammastar/!dtor
	return,1
end

pro lattice::cleanup
print,'lattice::cleanup'
end


pro lattice__define

define={lattice, $
		a:0.0d, $
		b:0.0d, $
		c:0.0d, $
		alpha:0.0d, $
		beta:0.0d, $
		gamma:0.0d, $
		astar:0.0d, $
		bstar:0.0d, $
		cstar:0.0d, $
		alphastar:0.0d, $
		betastar:0.0d, $
		gammastar:0.0d, $
		x:[0.0d,0.0d,0.0d], $
		y:[0.0d,0.0d,0.0d], $
		z:[0.0d,0.0d,0.0d] $
}


end